==> rjags_1.0.3-4.tar.gz: rjags/R/jags.R <==
print.jags <- function(x, ...)
{
  cat("JAGS model:\n\n")
  
  model <- x$model()
  for (i in 1:length(model)) {
    cat(model[i],"\n",sep="")
  }

  data <- x$data()
  full <- !sapply(lapply(data, is.na), any)
  if (any(full)) {
    cat("Fully observed variables:\n", names(data)[full], "\n")
  }
  part <- !full & !sapply(lapply(data, is.na), all)
  if (any(part)) {
    cat("Partially observed variables:\n", names(data)[part], "\n")
  }
}

jags.model <- function(file, data=sys.frame(sys.parent()), inits,
                       nchain = 1, n.adapt=1000)
{

    if (missing(file)) {
        stop("Model file name missing")
    }
    if (!file.exists(file)) {
        stop(paste("Model file \"", file, "\" not found", sep=""))
    }
    
    p <- .Call("make_console", PACKAGE="rjags") 
    .Call("check_model", p, file, PACKAGE="rjags")

    varnames <- .Call("get_variable_names", p, PACKAGE="rjags")
    if (is.environment(data)) {
        ##Get a list of numeric objects from the supplied environment
        data <- mget(varnames, envir=data, mode="numeric",
                     ifnotfound=list(NULL))
        ##Strip null entries
        data <- data[!sapply(data, is.null)]
    }
    else if (is.list(data)) {
        v <- names(data)
        if (is.null(v)) {
            stop("data must be a named list")
        }
        if (any(nchar(v)==0)) {
            stop("unnamed variables in data list")
        }
        if (any(duplicated(v))) {
            stop("Duplicated names in data list: ",
                 paste(v[duplicated(v)], collapse=" "))
        }
        relevant.variables <- names(data) %in% varnames
        data <- data[relevant.variables]
    }
    else {
        stop("data must be a list or environment")
    }
    
    .Call("compile", p, data, as.integer(nchain), TRUE, PACKAGE="rjags")

### Setting initial values

    if (!missing(inits)) {

        checkParameters <- function(inits) {
            if(!is.list(inits))
                return (FALSE)

            inames <- names(inits)
            if (is.null(inames) || any(nchar(inames) == 0))
                return (FALSE)

            if (any(duplicated(inames)))
                return (FALSE)
            
            if (any(inames==".RNG.name")) {
                rngname <- inits[[".RNG.name"]]
                if (!is.character(rngname) || length(rngname) != 1)
                    return (FALSE)
                inits[[".RNG.name"]] <- NULL
            }

            if (!all(sapply(inits, is.numeric)))
                return (FALSE)
            
            return (TRUE)
        }
        
        setParameters <- function(inits, chain) {
            if (!is.null(inits[[".RNG.name"]])) {
                .Call("set_rng_name", p, inits[[".RNG.name"]],
                      as.integer(chain), PACKAGE="rjags")
                inits[[".RNG.name"]] <- NULL
            }
            .Call("set_parameters", p, inits, as.integer(chain),
                  PACKAGE="rjags")
        }
        
        init.values <- vector("list", nchain)
        
        if (is.function(inits)) {
            if (any(names(formals(inits)) == "chain")) {
                for (i in 1:nchain) {
                    init.values[[i]] <- inits(chain=i)
                }
            }
            else {
                for (i in 1:nchain) {
                    init.values[[i]] <- inits()
                }
            }
        }
        else if (is.list(inits)) {

            if (checkParameters(inits)) {
                ## Replicate initial values for all chains
                for (i in 1:nchain) {
                    init.values[[i]] <- inits
                }
            }
            else {
                if (length(inits) != nchain) {
                    stop("Length mismatch in inits")
                }
                init.values <- inits
            }
        }
            
        for (i in 1:nchain) {
            if (!checkParameters(init.values[[i]])) {
                stop("Invalid parameters for chain ", i)
            }
            setParameters(init.values[[i]], i)
        }
    }

    .Call("initialize", p, PACKAGE="rjags")

    model.state <- .Call("get_state", p, PACKAGE="rjags")
    model.data <- .Call("get_data", p, PACKAGE="rjags")
    model.code <- readLines(file)
    model <- list("ptr" = function() {p},
                  "data" = function() {model.data},
                  "model" = function() {model.code},
                  "state" = function(internal=FALSE)
                  {
                      if(!internal) {
                          for(i in 1:nchain) {
                              model.state[[i]][[".RNG.state"]] <- NULL
                              model.state[[i]][[".RNG.name"]] <- NULL
                          }
                      }
                      return(model.state)
                  },
                  "nchain" = function()
                  {
                      .Call("get_nchain", p, PACKAGE="rjags")
                  },
                  "iter" = function()
                  {
                      .Call("get_iter", p, PACKAGE="rjags")
                  },
                  "update" = function(niter, by=niter/50, adapt=FALSE) {

                    adapting <- .Call("is_adapting", p, PACKAGE="rjags")
                    if (adapt & !adapting)
                      return(invisible(NULL))

                    if (niter <= 0)
                      stop("niter must be positive")
                    niter <- floor(niter)

                    if (by <= 0)
                      stop("by must be positive")
                    by <- ceiling(by)

                    if (interactive()) {
                      #Show progress bar
                      pb <- txtProgressBar(0, niter, style=3,width=50,
                                           char=ifelse(adapting,"+","*"))
                      n <- niter
                      while (n > 0) {
                        .Call("update", p, min(n,by), adapt, PACKAGE="rjags")
                        n <- n - by
                        setTxtProgressBar(pb, niter - n)
                        model.state <<- .Call("get_state", p, PACKAGE="rjags")
                      }
                      close(pb)
                    }
                    else {
                      #Suppress progress bar
                      .Call("update", p, niter, adapt, PACKAGE="rjags")
                      model.state <<- .Call("get_state", p, PACKAGE="rjags")
                    }
                    
                    if (adapting) {
                      if (!.Call("adapt_off", p, PACKAGE="rjags")) {
                        warning("Adaptation incomplete");
                      }
                    }
                    
                    invisible(NULL)
                  },
                  "recompile" = function() {
                      ## Clear the console
                      .Call("clear_console", p, PACKAGE="rjags")
                      p <<- .Call("make_console", PACKAGE="rjags")
                      ## Write the model to a temporary file so we can re-read it
                      mf <- tempfile()
                      writeLines(model.code, mf)
                      .Call("check_model", p, mf, PACKAGE="rjags")
                      unlink(mf)
                      ## Re-compile
                      .Call("compile", p, data, nchain, FALSE, PACKAGE="rjags")
                      ## Re-initialize
                      if (!is.null(model.state)) {
                          if (length(model.state) != nchain) {
                              stop("Incorrect number of chains in saved state")
                          }
                          for (i in 1:nchain) {
                              statei <- model.state[[i]]
                              rng <- statei[[".RNG.name"]]
                              if (!is.null(rng)) {
                                  .Call("set_rng_name", p, rng, i, PACKAGE="rjags")
                                  statei[[".RNG.name"]] <- NULL
                              }
                              .Call("set_parameters", p, statei, i, PACKAGE="rjags")
                          }
                          .Call("initialize", p, PACKAGE="rjags")
                          ## Redo adaptation
                          cat("Adapting\n")
                          .Call("update", p, n.adapt, TRUE, PACKAGE="rjags")
                          model.state <<- .Call("get_state", p, PACKAGE="rjags")
                      }
                      invisible(NULL)
                  })
    class(model) <- "jags"
    model$update(n.adapt, adapt=TRUE)
    return(model)
}

parse.varname <- function(varname) {

  ## Try to parse string of form "a" or "a[n,p:q,r]" where "a" is a
  ## variable name and n,p,q,r are integers

  v <- try(parse(text=varname, n=1), silent=TRUE)
  if (!is.expression(v) || length(v) != 1)
    return(NULL)

  v <- v[[1]]
  if (is.name(v)) {
    ##Full node array requested
    return(list(name=deparse(v)))
  }
  else if (is.call(v) && identical(deparse(v[[1]]), "[") && length(v) > 2) {
    ##Subset requested
    ndim <- length(v) - 2
    lower <- upper <- numeric(ndim)
    if (any(nchar(sapply(v, deparse)) == 0)) {
      ##We have to catch empty indices here or they will cause trouble
      ##below
      return(NULL)
    }
    for (i in 1:ndim) {
      index <- v[[i+2]]
      if (is.numeric(index)) {
        ##Single index
        lower[i] <- upper[i] <- index
      }
      else if (is.call(index) && length(index) == 3 &&
               identical(deparse(index[[1]]), ":") &&
               is.numeric(index[[2]]) && is.numeric(index[[3]]))
        {
          ##Index range
          lower[i] <- index[[2]]
          upper[i] <- index[[3]]
        }
      else return(NULL)
    }
    if (any(upper < lower))
      return (NULL)
    return(list(name = deparse(v[[2]]), lower=lower, upper=upper))
  }
  return(NULL)
}

parse.varnames <- function(varnames)
{
  names <- character(length(varnames))
  lower <- upper <- vector("list", length(varnames))
  for (i in seq(along=varnames)) {
    y <- parse.varname(varnames[i])
    if (is.null(y)) {
      stop(paste("Invalid variable subset", varnames[i]))
    }
    names[i] <- y$name
    if (!is.null(y$lower)) {
      lower[[i]] <- y$lower
    }
    if (!is.null(y$upper)) {
      upper[[i]] <- y$upper
    }
  }
  return(list(names=names, lower=lower, upper=upper))
}


jags.samples <-
  function(model, variable.names=NULL, n.iter, thin=1, type="trace")
{
    if (class(model) != "jags")
      stop("Invalid JAGS model")

    if (!is.null(variable.names)) {
       if (!is.character(variable.names))
         stop("variable.names must be a character vector")
    }
     
    if (!is.numeric(n.iter) || length(n.iter) != 1 || n.iter <= 0)
      stop("n.iter must be a positive integer")
    if (!is.character(type))
      stop("type must be a character vector")

    if (is.null(variable.names)) {
        .Call("set_default_monitors", model$ptr(), as.integer(thin),
              type, PACKAGE="rjags")
    }
    else {
      pn <- parse.varnames(variable.names)
      .Call("set_monitors", model$ptr(), pn$names, pn$lower, pn$upper,
            as.integer(thin), type, PACKAGE="rjags")
    }
    update(model, n.iter)
    ans <- .Call("get_monitored_values", model$ptr(), type, PACKAGE="rjags")
    for (i in seq(along=ans)) {
        class(ans[[i]]) <- "mcarray"
    }
    if (is.null(variable.names)) {
        .Call("clear_default_monitors", model$ptr(), type, PACKAGE="rjags")
    }
    else {
        for (i in seq(along=variable.names)) {
            .Call("clear_monitor", model$ptr(), pn$names[i], pn$lower[[i]],
                  pn$upper[[i]], type, PACKAGE="rjags")
        }
    }
    return(ans)
}

list.samplers <- function(model)
{
    if (!inherits(model, "jags")) {
        stop("not a jags model object")
    }
    .Call("get_samplers", model$ptr(), PACKAGE="rjags")
}


coda.names <- function(basename, dim)
{
    ## Utility function used to get the names of the individual elements
    ## of a node array
  
    if (prod(dim) == 1)
      return(basename)

    ##Default lower and upper limits
    ndim <- length(dim)
    lower <- rep(1, ndim)
    upper <- dim

    ##If the node name is a subset, we try to parse it to get the
    ##names of its elements. For example, if basename is "A[2:3]"
    ##we want to return names "A[2]", "A[3]" not "A[2:3][1]", "A[2:3][2]".
    pn <- parse.varname(basename)
    if (!is.null(pn) && !is.null(pn$lower) && !is.null(pn$upper)) {
        if (length(pn$lower) == length(pn$upper)) {
            dim2 <- pn$upper - pn$lower + 1
            if (isTRUE(all.equal(dim[dim!=1], dim2[dim2!=1],
                                 check.attributes=FALSE))) {
                basename <- pn$name
                lower <- pn$lower
                upper <- pn$upper
                ndim <- length(dim2)
            }
        }
    }
    
    indices <- as.character(lower[1]:upper[1])
    if (ndim > 1) {
        for (i in 2:ndim) {
            indices <- outer(indices, lower[i]:upper[i], FUN=paste, sep=",")
        }
    }
    paste(basename,"[",as.vector(indices),"]",sep="")
}

nchain <- function(model)
{
    if (!inherits(model, "jags"))
      stop("Invalid JAGS model object in nchain")
    
    .Call("get_nchain", model$ptr(), PACKAGE="rjags")
}

coda.samples <- function(model, variable.names=NULL, n.iter, thin=1)
{
    start <- model$iter() + thin
    out <- jags.samples(model, variable.names, n.iter, thin, type="trace")

    ans <- vector("list", nchain(model))
    for (ch in 1:nchain(model)) {
        ans.ch <- vector("list", length(out))

        vnames.ch <- NULL
        for (i in seq(along=out)) {

            varname <- names(out)[[i]]
            d <- dim(out[[i]])
            if (length(d) < 3) {
                stop("Invalid dimensions for sampled output")
            }
            vardim <- d[1:(length(d)-2)]
            nvar <- prod(vardim)
            niter <- d[length(d) - 1]        
            nchain <- d[length(d)]
            
            values <- as.vector(out[[i]])
            var.i <- matrix(NA, nrow=niter, ncol=nvar)
            for (j in 1:nvar) {
                var.i[,j] <- values[j + (0:(niter-1))*nvar + (ch-1)*niter*nvar]
            }
            vnames.ch <- c(vnames.ch, coda.names(varname, vardim))
            ans.ch[[i]] <- var.i
        }
        
        ans.ch <- as.matrix(data.frame(ans.ch))
        colnames(ans.ch) <- vnames.ch
        ans[[ch]] <- mcmc(ans.ch, start=start, thin=thin)
    }
    mcmc.list(ans)
}

jags.module <- function(names, path)
{
    if (missing(path)) {
        path = getOption("jags.moddir")
        if (is.null(path)) {
            stop("option jags.moddir is not set")
        }
    }
    
    cat("loading JAGS module\n")
    for (i in 1:length(names)) {
        cat("   ", names[i], "\n", sep="")
        file <- file.path(path,
                          paste(names[i], .Platform$dynlib.ext, sep=""))
        if (!file.exists(file)) {
            stop("Cannot load ", file)
        }
        dyn.load(file)
    }
}

==> rjags_1.0.3-5.tar.gz: rjags/R/jags.R <==
print.jags <- function(x, ...)
{
  cat("JAGS model:\n\n")
  
  model <- x$model()
  for (i in 1:length(model)) {
    cat(model[i],"\n",sep="")
  }

  data <- x$data()
  full <- !sapply(lapply(data, is.na), any)
  if (any(full)) {
    cat("Fully observed variables:\n", names(data)[full], "\n")
  }
  part <- !full & !sapply(lapply(data, is.na), all)
  if (any(part)) {
    cat("Partially observed variables:\n", names(data)[part], "\n")
  }
}

jags.model <- function(file, data=sys.frame(sys.parent()), inits,
                       n.chains = 1, n.adapt=1000, nchain)
{

    if (missing(file)) {
        stop("Model file name missing")
    }
    if (!file.exists(file)) {
        stop(paste("Model file \"", file, "\" not found", sep=""))
    }
    if (!missing(nchain)) {
        warning("Argument nchain in jags.model is deprecated. Use n.chains.")
        if (missing(n.chains)) {
            n.chains = nchain
        }
    }
    
    p <- .Call("make_console", PACKAGE="rjags") 
    .Call("check_model", p, file, PACKAGE="rjags")

    varnames <- .Call("get_variable_names", p, PACKAGE="rjags")
    if (is.environment(data)) {
        ##Get a list of numeric objects from the supplied environment
        data <- mget(varnames, envir=data, mode="numeric",
                     ifnotfound=list(NULL))
        ##Strip null entries
        data <- data[!sapply(data, is.null)]
    }
    else if (is.list(data)) {
        v <- names(data)
        if (is.null(v)) {
            stop("data must be a named list")
        }
        if (any(nchar(v)==0)) {
            stop("unnamed variables in data list")
        }
        if (any(duplicated(v))) {
            stop("Duplicated names in data list: ",
                 paste(v[duplicated(v)], collapse=" "))
        }
        relevant.variables <- names(data) %in% varnames
        data <- data[relevant.variables]
    }
    else {
        stop("data must be a list or environment")
    }
    
    .Call("compile", p, data, as.integer(n.chains), TRUE, PACKAGE="rjags")

### Setting initial values

    if (!missing(inits)) {

        checkParameters <- function(inits) {
            if(!is.list(inits))
                return (FALSE)

            inames <- names(inits)
            if (is.null(inames) || any(nchar(inames) == 0))
                return (FALSE)

            if (any(duplicated(inames)))
                return (FALSE)
            
            if (any(inames==".RNG.name")) {
                rngname <- inits[[".RNG.name"]]
                if (!is.character(rngname) || length(rngname) != 1)
                    return (FALSE)
                inits[[".RNG.name"]] <- NULL
            }

            if (!all(sapply(inits, is.numeric)))
                return (FALSE)
            
            return (TRUE)
        }
        
        setParameters <- function(inits, chain) {
            if (!is.null(inits[[".RNG.name"]])) {
                .Call("set_rng_name", p, inits[[".RNG.name"]],
                      as.integer(chain), PACKAGE="rjags")
                inits[[".RNG.name"]] <- NULL
            }
            .Call("set_parameters", p, inits, as.integer(chain),
                  PACKAGE="rjags")
        }
        
        init.values <- vector("list", n.chains)
        
        if (is.function(inits)) {
            if (any(names(formals(inits)) == "chain")) {
                for (i in 1:n.chains) {
                    init.values[[i]] <- inits(chain=i)
                }
            }
            else {
                for (i in 1:n.chains) {
                    init.values[[i]] <- inits()
                }
            }
        }
        else if (is.list(inits)) {

            if (checkParameters(inits)) {
                ## Replicate initial values for all chains
                for (i in 1:n.chains) {
                    init.values[[i]] <- inits
                }
            }
            else {
                if (length(inits) != n.chains) {
                    stop("Length mismatch in inits")
                }
                init.values <- inits
            }
        }
            
        for (i in 1:n.chains) {
            if (!checkParameters(init.values[[i]])) {
                stop("Invalid parameters for chain ", i)
            }
            setParameters(init.values[[i]], i)
        }
    }

    .Call("initialize", p, PACKAGE="rjags")

    model.state <- .Call("get_state", p, PACKAGE="rjags")
    model.data <- .Call("get_data", p, PACKAGE="rjags")
    model.code <- readLines(file)
    model <- list("ptr" = function() {p},
                  "data" = function() {model.data},
                  "model" = function() {model.code},
                  "state" = function(internal=FALSE)
                  {
                      if(!internal) {
                          for(i in 1:n.chains) {
                              model.state[[i]][[".RNG.state"]] <- NULL
                              model.state[[i]][[".RNG.name"]] <- NULL
                          }
                      }
                      return(model.state)
                  },
                  "nchain" = function()
                  {
                      .Call("get_nchain", p, PACKAGE="rjags")
                  },
                  "iter" = function()
                  {
                      .Call("get_iter", p, PACKAGE="rjags")
                  },
                  "update" = function(n.iter, by=n.iter/50, adapt=FALSE) {

                    adapting <- .Call("is_adapting", p, PACKAGE="rjags")
                    if (adapt & !adapting)
                      return(invisible(NULL))

                    if (n.iter <= 0)
                      stop("n.iter must be positive")
                    n.iter <- floor(n.iter)

                    if (by <= 0)
                      stop("by must be positive")
                    by <- ceiling(by)

                    if (interactive()) {
                      #Show progress bar
                      pb <- txtProgressBar(0, n.iter, style=3,width=50,
                                           char=ifelse(adapting,"+","*"))
                      n <- n.iter
                      while (n > 0) {
                        .Call("update", p, min(n,by), adapt, PACKAGE="rjags")
                        n <- n - by
                        setTxtProgressBar(pb, n.iter - n)
                        model.state <<- .Call("get_state", p, PACKAGE="rjags")
                      }
                      close(pb)
                    }
                    else {
                      #Suppress progress bar
                      .Call("update", p, n.iter, adapt, PACKAGE="rjags")
                      model.state <<- .Call("get_state", p, PACKAGE="rjags")
                    }
                    
                    if (adapting) {
                      if (!.Call("adapt_off", p, PACKAGE="rjags")) {
                        warning("Adaptation incomplete");
                      }
                    }
                    
                    invisible(NULL)
                  },
                  "recompile" = function() {
                      ## Clear the console
                      .Call("clear_console", p, PACKAGE="rjags")
                      p <<- .Call("make_console", PACKAGE="rjags")
                      ## Write the model to a temporary file so we can re-read it
                      mf <- tempfile()
                      writeLines(model.code, mf)
                      .Call("check_model", p, mf, PACKAGE="rjags")
                      unlink(mf)
                      ## Re-compile
                      .Call("compile", p, data, n.chains, FALSE, PACKAGE="rjags")
                      ## Re-initialize
                      if (!is.null(model.state)) {
                          if (length(model.state) != n.chains) {
                              stop("Incorrect number of chains in saved state")
                          }
                          for (i in 1:n.chains) {
                              statei <- model.state[[i]]
                              rng <- statei[[".RNG.name"]]
                              if (!is.null(rng)) {
                                  .Call("set_rng_name", p, rng, i, PACKAGE="rjags")
                                  statei[[".RNG.name"]] <- NULL
                              }
                              .Call("set_parameters", p, statei, i, PACKAGE="rjags")
                          }
                          .Call("initialize", p, PACKAGE="rjags")
                          ## Redo adaptation
                          cat("Adapting\n")
                          .Call("update", p, n.adapt, TRUE, PACKAGE="rjags")
                          model.state <<- .Call("get_state", p, PACKAGE="rjags")
                      }
                      invisible(NULL)
                  })
    class(model) <- "jags"
    model$update(n.adapt, adapt=TRUE)
    return(model)
}

parse.varname <- function(varname) {

  ## Try to parse string of form "a" or "a[n,p:q,r]" where "a" is a
  ## variable name and n,p,q,r are integers

  v <- try(parse(text=varname, n=1), silent=TRUE)
  if (!is.expression(v) || length(v) != 1)
    return(NULL)

  v <- v[[1]]
  if (is.name(v)) {
    ##Full node array requested
    return(list(name=deparse(v)))
  }
  else if (is.call(v) && identical(deparse(v[[1]]), "[") && length(v) > 2) {
    ##Subset requested
    ndim <- length(v) - 2
    lower <- upper <- numeric(ndim)
    if (any(nchar(sapply(v, deparse)) == 0)) {
      ##We have to catch empty indices here or they will cause trouble
      ##below
      return(NULL)
    }
    for (i in 1:ndim) {
      index <- v[[i+2]]
      if (is.numeric(index)) {
        ##Single index
        lower[i] <- upper[i] <- index
      }
      else if (is.call(index) && length(index) == 3 &&
               identical(deparse(index[[1]]), ":") &&
               is.numeric(index[[2]]) && is.numeric(index[[3]]))
        {
          ##Index range
          lower[i] <- index[[2]]
          upper[i] <- index[[3]]
        }
      else return(NULL)
    }
    if (any(upper < lower))
      return (NULL)
    return(list(name = deparse(v[[2]]), lower=lower, upper=upper))
  }
  return(NULL)
}

parse.varnames <- function(varnames)
{
  names <- character(length(varnames))
  lower <- upper <- vector("list", length(varnames))
  for (i in seq(along=varnames)) {
    y <- parse.varname(varnames[i])
    if (is.null(y)) {
      stop(paste("Invalid variable subset", varnames[i]))
    }
    names[i] <- y$name
    if (!is.null(y$lower)) {
      lower[[i]] <- y$lower
    }
    if (!is.null(y$upper)) {
      upper[[i]] <- y$upper
    }
  }
  return(list(names=names, lower=lower, upper=upper))
}


jags.samples <-
  function(model, variable.names=NULL, n.iter, thin=1, type="trace")
{
    if (class(model) != "jags")
      stop("Invalid JAGS model")

    if (!is.null(variable.names)) {
       if (!is.character(variable.names))
         stop("variable.names must be a character vector")
    }
     
    if (!is.numeric(n.iter) || length(n.iter) != 1 || n.iter <= 0)
      stop("n.iter must be a positive integer")
    if (!is.character(type))
      stop("type must be a character vector")

    if (is.null(variable.names)) {
        .Call("set_default_monitors", model$ptr(), as.integer(thin),
              type, PACKAGE="rjags")
    }
    else {
      pn <- parse.varnames(variable.names)
      .Call("set_monitors", model$ptr(), pn$names, pn$lower, pn$upper,
            as.integer(thin), type, PACKAGE="rjags")
    }
    update(model, n.iter)
    ans <- .Call("get_monitored_values", model$ptr(), type, PACKAGE="rjags")
    for (i in seq(along=ans)) {
        class(ans[[i]]) <- "mcarray"
    }
    if (is.null(variable.names)) {
        .Call("clear_default_monitors", model$ptr(), type, PACKAGE="rjags")
    }
    else {
        for (i in seq(along=variable.names)) {
            .Call("clear_monitor", model$ptr(), pn$names[i], pn$lower[[i]],
                  pn$upper[[i]], type, PACKAGE="rjags")
        }
    }
    return(ans)
}

list.samplers <- function(model)
{
    if (!inherits(model, "jags")) {
        stop("not a jags model object")
    }
    .Call("get_samplers", model$ptr(), PACKAGE="rjags")
}


coda.names <- function(basename, dim)
{
    ## Utility function used to get the names of the individual elements
    ## of a node array
  
    if (prod(dim) == 1)
      return(basename)

    ##Default lower and upper limits
    ndim <- length(dim)
    lower <- rep(1, ndim)
    upper <- dim

    ##If the node name is a subset, we try to parse it to get the
    ##names of its elements. For example, if basename is "A[2:3]"
    ##we want to return names "A[2]", "A[3]" not "A[2:3][1]", "A[2:3][2]".
    pn <- parse.varname(basename)
    if (!is.null(pn) && !is.null(pn$lower) && !is.null(pn$upper)) {
        if (length(pn$lower) == length(pn$upper)) {
            dim2 <- pn$upper - pn$lower + 1
            if (isTRUE(all.equal(dim[dim!=1], dim2[dim2!=1],
                                 check.attributes=FALSE))) {
                basename <- pn$name
                lower <- pn$lower
                upper <- pn$upper
                ndim <- length(dim2)
            }
        }
    }
    
    indices <- as.character(lower[1]:upper[1])
    if (ndim > 1) {
        for (i in 2:ndim) {
            indices <- outer(indices, lower[i]:upper[i], FUN=paste, sep=",")
        }
    }
    paste(basename,"[",as.vector(indices),"]",sep="")
}

nchain <- function(model)
{
    if (!inherits(model, "jags"))
      stop("Invalid JAGS model object in nchain")
    
    .Call("get_nchain", model$ptr(), PACKAGE="rjags")
}

coda.samples <- function(model, variable.names=NULL, n.iter, thin=1)
{
    start <- model$iter() + thin
    out <- jags.samples(model, variable.names, n.iter, thin, type="trace")

    ans <- vector("list", nchain(model))
    for (ch in 1:nchain(model)) {
        ans.ch <- vector("list", length(out))

        vnames.ch <- NULL
        for (i in seq(along=out)) {

            varname <- names(out)[[i]]
            d <- dim(out[[i]])
            if (length(d) < 3) {
                stop("Invalid dimensions for sampled output")
            }
            vardim <- d[1:(length(d)-2)]
            nvar <- prod(vardim)
            niter <- d[length(d) - 1]        
            nchain <- d[length(d)]
            
            values <- as.vector(out[[i]])
            var.i <- matrix(NA, nrow=niter, ncol=nvar)
            for (j in 1:nvar) {
                var.i[,j] <- values[j + (0:(niter-1))*nvar + (ch-1)*niter*nvar]
            }
            vnames.ch <- c(vnames.ch, coda.names(varname, vardim))
            ans.ch[[i]] <- var.i
        }
        
        ans.ch <- as.matrix(data.frame(ans.ch))
        colnames(ans.ch) <- vnames.ch
        ans[[ch]] <- mcmc(ans.ch, start=start, thin=thin)
    }
    mcmc.list(ans)
}

jags.module <- function(names, path)
{
    if (missing(path)) {
        path = getOption("jags.moddir")
        if (is.null(path)) {
            stop("option jags.moddir is not set")
        }
    }
    
    cat("loading JAGS module\n")
    for (i in 1:length(names)) {
        cat("   ", names[i], "\n", sep="")
        file <- file.path(path,
                          paste(names[i], .Platform$dynlib.ext, sep=""))
        if (!file.exists(file)) {
            stop("Cannot load ", file)
        }
        dyn.load(file)
    }
}
